﻿package com
{
	import com.adobe.serialization.json.*;
	import com.greensock.TweenLite;
	import com.greensock.easing.*;

	import flash.display.*;
	import flash.events.*;
	import flash.net.*;
	import flash.ui.*;
	import flash.utils.*;

	public class Game extends Sprite
	{

		private var goldenEggsData:Object;
		private var crackedEggsData:Object;
		private var goldenEggsArray:Array = new Array();
		private var crackedEggsArray:Array = new Array();
		private var goldenEggsCount:int = 0;
		private var crackedEggsCount:int = 0;

		private var speed:int = 4;
		private var nest:Nest = new Nest();
		private var eggs:Array = new Array();
		private var tweetsOnScreen:Array = new Array();
		private var score:Number = 0;
		private var difficulty:Number = 3;
		private var ended:Boolean = false;

		private var rightKey:Boolean = false;
		private var leftKey:Boolean = false;

		private var tweetsBar:TweetsBar = new TweetsBar();
		public var addedToStage:Boolean = false;

		public function Game()
		{
			// tweets loader
			var loader:URLLoader = new URLLoader();
			loader.addEventListener(Event.COMPLETE, loadCrackedEggs);
			loader.addEventListener(IOErrorEvent.IO_ERROR, errorHandler);

			// construct the URL ors = what we are looking for rpp = the amount of tweets that we want
			var url:String = "http://search.twitter.com/search.json?lang=en&q=golden%20eggs&rpp=50";

			loader.load(new URLRequest(url));

			this.addEventListener(Event.ADDED, init);
		}
		
		private function loadCrackedEggs(e:Event):void
		{
			goldenEggsData = JSON.decode(e.currentTarget.data);

			// tweets loader
			var loader:URLLoader = new URLLoader();
			loader.addEventListener(Event.COMPLETE, startGame);
			loader.addEventListener(IOErrorEvent.IO_ERROR, errorHandler);

			// construct the URL ors = what we are looking for rpp = the amount of tweets that we want
			var url:String = "http://search.twitter.com/search.json?q=cracked%20eggs&lang=en&rpp=50";

			loader.load(new URLRequest(url));
		}

		private function errorHandler(e:IOErrorEvent):void
		{
			dispatchEvent(new GameEvent(GameEvent.TWEETS_ERROR));
		}

		private function startGame(e:Event):void
		{
			crackedEggsData = JSON.decode(e.currentTarget.data);

			for (var i:int = 0; i < goldenEggsData.results.length; i++)
			{
				var goldenEgg:Egg = new Egg();
				goldenEgg.tweet = goldenEggsData.results[i].text;
				goldenEgg.user = goldenEggsData.results[i].from_user;
				goldenEgg.userImg = goldenEggsData.results[i].profile_image_url;
				goldenEgg.type = "Golden Egg";

				goldenEggsArray.push(goldenEgg);
			}

			for (var x:int = 0; x < crackedEggsData.results.length; x++)
			{
				var crackedEgg:Egg = new Egg();
				crackedEgg.tweet = crackedEggsData.results[x].text;
				crackedEgg.user = crackedEggsData.results[x].from_user;
				crackedEgg.userImg = crackedEggsData.results[x].profile_image_url;
				crackedEgg.type = "Cracked Egg";

				crackedEggsArray.push(crackedEgg);
			}

			dispatchEvent(new GameEvent(GameEvent.TWEETS_READY));
		}
		
		private function init(e:Event):void
		{
			this.removeEventListener(Event.ADDED, init);

			addedToStage = true;
			nest.x = 10;
			nest.y = 497;
			nest.alpha = 0;
			TweenLite.to(nest, .5,{alpha:1});
			addChild(nest);

			tweetsBar.x = 350;
			tweetsBar.alpha = 0;
			TweenLite.to(tweetsBar, .6,{alpha:1});
			addChild(tweetsBar);

			stage.addEventListener(Event.ENTER_FRAME, updateGame);
			stage.addEventListener(KeyboardEvent.KEY_DOWN, keyDownHandler);
			stage.addEventListener(KeyboardEvent.KEY_UP, keyUpHandler);

			createBird();

			TweenLite.delayedCall(10, changeDifficulty);
		}

		private function createBird():void
		{
			var bird:Bird = new Bird();
			var time:Number = (Math.random()*1.3)+.5;

			bird.x = -100;
			bird.y = (Math.random()*60) + 50;

			TweenLite.to(bird, 5, {x: 600, onComplete:killObject, onCompleteParams:[bird]});
			
			addChild(bird);

			if (! ended)
			{
				TweenLite.delayedCall(time, releaseEgg, [bird]);
				TweenLite.delayedCall(Math.random()*difficulty, createBird);
			}
			else
			{
				stage.removeEventListener(Event.ENTER_FRAME, updateGame);
				
				TweenLite.killTweensOf(releaseEgg);
				
				if (eggs.length > 0)
				{
					for (var i:int = 0; i < eggs.length; i)
					{
						killObject(eggs[i]);
						eggs.shift();
					}
				}

				TweenLite.to(nest, .6,{y:700});
				
				showTweetScore();
			}
		}

		private function releaseEgg(b:Bird):void
		{
			var r:Number = Math.random();
			var egg:Egg;

			if (r > 0.45)
			{
				egg = goldenEggsArray[goldenEggsCount];
				goldenEggsCount++;
			}
			else
			{
				egg = crackedEggsArray[crackedEggsCount];
				crackedEggsCount++;
			}

			egg.x = b.x;
			egg.y = b.y;

			addChild(egg);
			eggs.push(egg);
		}

		private function changeDifficulty():void
		{
			if (difficulty == 3)
			{
				difficulty = 2;
				speed = 5;

				TweenLite.delayedCall(10, changeDifficulty);
				return;
			}
			if (difficulty == 2)
			{
				difficulty = 1.5;
				speed = 7;

				TweenLite.delayedCall(5, changeDifficulty);
				return;
			}
			if (difficulty == 1.5)
			{
				difficulty = 1;
				speed = 8;

				TweenLite.delayedCall(5, changeDifficulty);
				return;
			}
			if (difficulty == 1)
			{
				ended = true;
				return;
			}
		}
		
		private function keyDownHandler(e:KeyboardEvent):void
		{
			if (e.keyCode == 37)
			{
				leftKey = true;
			}

			if (e.keyCode == 39)
			{
				rightKey = true;
			}
		}

		private function keyUpHandler(e:KeyboardEvent):void
		{
			if (e.keyCode == 37)
			{
				leftKey = false;
			}

			if (e.keyCode == 39)
			{
				rightKey = false;
			}
		}
		
		private function showTweetScore():void
		{
			var tweetMessage:TweetMessage = new TweetMessage();
			tweetMessage.x = 10;
			tweetMessage.y = 270;
			tweetMessage.message_text.text = "You just scored "+score.toString()+" point on Golden Eggs!";
			tweetMessage.embedFonts = true;
			addChild(tweetMessage);
			
			TweenLite.from(tweetMessage, .6, {alpha:0});
			
			tweetMessage.tweet_btn.buttonMode = true;
			tweetMessage.tweet_btn.addEventListener(MouseEvent.CLICK, shareScore);
		}
		
		private function shareScore(e:MouseEvent):void
		{
			var url:String = encodeURI("https://twitter.com/share?text=I just scored "+score.toString()+" points in Golden Eggs!, try to beat me!&url=http://www.jsCampos.com/GoldenEggs&via=envatoactive");
			navigateToURL(new URLRequest(url), "_blank");
		}

		private function killObject(ob:Sprite):void
		{
			ob.parent.removeChild(ob);
			ob = null;
		}

		public function killGame():void
		{
			if(addedToStage)
			{
				stage.removeEventListener(Event.ENTER_FRAME, updateGame);
				stage.removeEventListener(KeyboardEvent.KEY_DOWN, keyDownHandler);
				stage.removeEventListener(KeyboardEvent.KEY_UP, keyUpHandler);
			}
		}

		private function updateGame(e:Event):void
		{
			if (nest.x < 250 && rightKey)
			{
				nest.x +=  5;
			}

			if (nest.x > 9 && leftKey)
			{
				nest.x -=  5;
			}

			if (eggs.length > 0)
			{
				for (var i:int = 0; i < eggs.length; i++)
				{
					eggs[i].y +=  speed;
					if (eggs[i].hitTestObject(nest.hit))
					{
						if (eggs[i].type == "Golden Egg")
						{
							score +=  100;
						}
						else
						{
							score -=  80;
						}

						var gameEvent:GameEvent = new GameEvent(GameEvent.SCORE_CHANGE,score.toString());
						dispatchEvent(gameEvent);

						var tweet:TweetHolder = new TweetHolder();
						tweet.user_text.text = eggs[i].user;
						tweet.tweet_text.text = eggs[i].tweet;
						tweet.userImg = eggs[i].userImg;
						tweet.x = 5;

						tweetsBar.addChild(tweet);

						if (tweetsOnScreen.length > 0)
						{
							for (var a:int = 0; a < tweetsOnScreen.length; a++)
							{
								tweetsOnScreen[a].y = tweetsOnScreen[a].y - tweet.height;
							}

							if (tweetsOnScreen[0].y < 100)
							{
								TweenLite.to(tweetsOnScreen[0], .5,{alpha:0, y:"-100", onComplete:tweetsOnScreen[0].parent.removeChild, onCompleteParams:[tweetsOnScreen[0]]});
								tweetsOnScreen.shift();
							}

							if (tweetsOnScreen[0].y < 100)
							{
								tweetsOnScreen[0].parent.removeChild(tweetsOnScreen[0]);
								tweetsOnScreen.shift();
							}
						}

						tweetsOnScreen.push(tweet);

						tweet.alpha = 1;

						tweet.y = 600 - tweet.height - 10;

						killObject(eggs[i]);
						eggs.splice(i, 1);
					}
					else if (eggs[i].y > 600)
					{
						killObject(eggs[i]);
						eggs.splice(i, 1);
					}
				}
			}
		}
	}
}